----------------------------------------------------------------------------------
-- Company:    Nabla Designs s.l.
-- Engineer:   F.T.
-- 
-- Create Date:     September 13th 2011 
-- Design Name: 
-- Module Name:    internal_leds_pushbuttons - Behavioral 
-- Project Name: 
-- Target Devices: SIMPlugIN LX45 board - Spartan 6 LX45 -2 
-- Tool versions: ISE 12.4
-- Description: exercises on board leds, switches and pushbuttons
--
-- Dependencies: 
--
-- Revision: 
-- Revision 1.0 - first realease
-- Additional Comments: 
--  Refer to SIMPlugIN-LX45 user manual

-- FUNCTIONALITY to test on board pushbuttons, switches and leds
--   * pushbutton 0 is reset for all the funcionality
--   * when any pushbutton is pressed each led reflects the state of pushbuttons 
--      otherwise
--         when any switch is on each led reflects the state of the switchs 
--           otherwise 
--             the leds follow the state of 4 bit binay counter



----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


entity internal_leds_pushbuttons is
   Port (
      clk_25mhz      : in   STD_LOGIC ; -- onboard 25 MHz oscillator
      reset_pow_on   : in   STD_LOGIC ; -- user power on reset (active high)  , it is also activated by pressing push buttom 0

      push_buttom_1_n  : in   STD_LOGIC ;  -- onboard user pushbutton 1
      push_buttom_2_n  : in   STD_LOGIC ;  -- onboard user pushbutton 2
      push_buttom_3_n  : in   STD_LOGIC ;  -- onboard user pushbutton 3
      push_buttom_4_n  : in   STD_LOGIC ;  -- onboard user pushbutton 4

      led1_n         : out  STD_LOGIC ; -- onboard user LED1
      led2_n         : out  STD_LOGIC ; -- onboard user LED2
      led3_n         : out  STD_LOGIC ; -- onboard user LED3
      led4_n         : out  STD_LOGIC ; -- onboard user LED4

      switch_1_n      : in   STD_LOGIC ; -- onboard user switch - position 1
      switch_2_n      : in   STD_LOGIC ; -- onboard user switch - position 2
      switch_3_n      : in   STD_LOGIC ; -- onboard user switch - position 3
      switch_4_n      : in   STD_LOGIC ; -- onboard user switch - position 4

      CONSOLE_CTS_N       : in  STD_LOGIC ;  -- console CTS (fpga in)
      CONSOLE_RTS_N       : out STD_LOGIC ;  -- console RTS (fpga out)   
      CONSOLE_RXD         : in  STD_LOGIC ;  -- console RxD (fpga in)     
      CONSOLE_TXD         : out STD_LOGIC ;  -- console TxD (fpga out)     
      CONSOLE_USB_PRESENT : in  STD_LOGIC   -- console usb present (input, active high)


-- The following pins are NOT used in the present project
-- ======================================================

--CLK_user : in   STD_LOGIC ;   -- optional user clock. Not used in this project

-- user connector pins. Not used in this project
--con1_05 : out  STD_LOGIC ;
--con1_06 : out  STD_LOGIC ;
--con1_07 : out  STD_LOGIC ;
--con1_08 : out  STD_LOGIC ;
--con1_11 : out  STD_LOGIC ;
--con1_12 : out  STD_LOGIC ;
--con1_13 : out  STD_LOGIC ;
--con1_14 : out  STD_LOGIC ;
--con1_17 : out  STD_LOGIC ;
--con1_18 : out  STD_LOGIC ;
--con1_19 : out  STD_LOGIC ;
--con1_20 : out  STD_LOGIC ;
--con1_23 : out  STD_LOGIC ;
--con1_24 : out  STD_LOGIC ;
--con1_25 : out  STD_LOGIC ;
--con1_26 : out  STD_LOGIC ;
--con1_29 : out  STD_LOGIC ;
--con1_30 : out  STD_LOGIC ;
--con1_31 : out  STD_LOGIC ;
--con1_32 : out  STD_LOGIC ;

--con2_05 : out  STD_LOGIC ;
--con2_06 : out  STD_LOGIC ;
--con2_07 : out  STD_LOGIC ;
--con2_08 : out  STD_LOGIC ;
--con2_11 : out  STD_LOGIC ;
--con2_12 : out  STD_LOGIC ;
--con2_13 : out  STD_LOGIC ;
--con2_14 : out  STD_LOGIC ;
--con2_17 : out  STD_LOGIC ;
--con2_18 : out  STD_LOGIC ;
--con2_19 : out  STD_LOGIC ;
--con2_20 : out  STD_LOGIC ;
--con2_23 : out  STD_LOGIC ;
--con2_24 : out  STD_LOGIC ;
--con2_25 : out  STD_LOGIC ;
--con2_26 : out  STD_LOGIC ;
--con2_29 : out  STD_LOGIC ;
--con2_30 : out  STD_LOGIC ;
--con2_31 : out  STD_LOGIC ;
--con2_32 : out  STD_LOGIC ;

--con3_05 : out  STD_LOGIC ;
--con3_06 : out  STD_LOGIC ;
--con3_07 : out  STD_LOGIC ;
--con3_08 : out  STD_LOGIC ;
--con3_11 : out  STD_LOGIC ;
--con3_12 : out  STD_LOGIC ;
--con3_13 : out  STD_LOGIC ;
--con3_14 : out  STD_LOGIC ;
--con3_17 : out  STD_LOGIC ;
--con3_18 : out  STD_LOGIC ;
--con3_19 : out  STD_LOGIC ;
--con3_20 : out  STD_LOGIC ;
--con3_23 : out  STD_LOGIC ;
--con3_24 : out  STD_LOGIC ;
--con3_25 : out  STD_LOGIC ;
--con3_26 : out  STD_LOGIC ;
--con3_29 : out  STD_LOGIC ;
--con3_30 : out  STD_LOGIC ;
--con3_31 : out  STD_LOGIC ;
--con3_32 : out  STD_LOGIC ;

--con4_05 : out  STD_LOGIC ;
--con4_06 : out  STD_LOGIC ;
--con4_07 : out  STD_LOGIC ;
--con4_08 : out  STD_LOGIC ;
--con4_11 : out  STD_LOGIC ;
--con4_12 : out  STD_LOGIC ;
--con4_13 : out  STD_LOGIC ;
--con4_14 : out  STD_LOGIC ;
--con4_17 : out  STD_LOGIC ;
--con4_18 : out  STD_LOGIC ;
--con4_19 : out  STD_LOGIC ;
--con4_20 : out  STD_LOGIC ;
--con4_23 : out  STD_LOGIC ;
--con4_24 : out  STD_LOGIC ;
--con4_25 : out  STD_LOGIC ;
--con4_26 : out  STD_LOGIC ;
--con4_29 : out  STD_LOGIC ;
--con4_30 : out  STD_LOGIC ;
--con4_31 : out  STD_LOGIC ;
--con4_32 : out  STD_LOGIC ;

--con5_05 : out  STD_LOGIC ;
--con5_06 : out  STD_LOGIC ;
--con5_07 : out  STD_LOGIC ;
--con5_08 : out  STD_LOGIC ;
--con5_11 : out  STD_LOGIC ;
--con5_12 : out  STD_LOGIC ;
--con5_13 : out  STD_LOGIC ;
--con5_14 : out  STD_LOGIC ;
--con5_17 : out  STD_LOGIC ;
--con5_18 : out  STD_LOGIC ;
--con5_19 : out  STD_LOGIC ;
--con5_20 : out  STD_LOGIC ;
--con5_23 : out  STD_LOGIC ;
--con5_24 : out  STD_LOGIC ;
--con5_25 : out  STD_LOGIC ;
--con5_26 : out  STD_LOGIC ;
--con5_29 : out  STD_LOGIC ;
--con5_30 : out  STD_LOGIC ;
--con5_31 : out  STD_LOGIC ;
--con5_32 : out  STD_LOGIC ;

--con6_05 : out  STD_LOGIC ;
--con6_06 : out  STD_LOGIC ;
--con6_07 : out  STD_LOGIC ;
--con6_08 : out  STD_LOGIC ;
--con6_11 : out  STD_LOGIC ;
--con6_12 : out  STD_LOGIC ;
--con6_13 : out  STD_LOGIC ;
--con6_14 : out  STD_LOGIC ;
--con6_17 : out  STD_LOGIC ;
--con6_18 : out  STD_LOGIC ;
--con6_19 : out  STD_LOGIC ;
--con6_20 : out  STD_LOGIC ;
--con6_23 : out  STD_LOGIC ;
--con6_24 : out  STD_LOGIC ;
--con6_25 : out  STD_LOGIC ;
--con6_26 : out  STD_LOGIC ;
--con6_29 : out  STD_LOGIC ;
--con6_30 : out  STD_LOGIC ;
--con6_31 : out  STD_LOGIC ;
--con6_32 : out  STD_LOGIC ;

--con7_05 : out  STD_LOGIC ;
--con7_06 : out  STD_LOGIC ;
--con7_07 : out  STD_LOGIC ;
--con7_08 : out  STD_LOGIC ;
--con7_11 : out  STD_LOGIC ;
--con7_12 : out  STD_LOGIC ;
--con7_13 : out  STD_LOGIC ;
--con7_14 : out  STD_LOGIC ;
--con7_17 : out  STD_LOGIC ;
--con7_18 : out  STD_LOGIC ;
--con7_19 : out  STD_LOGIC ;
--con7_20 : out  STD_LOGIC ;
--con7_23 : out  STD_LOGIC ;
--con7_24 : out  STD_LOGIC ;
--con7_25 : out  STD_LOGIC ;
--con7_26 : out  STD_LOGIC ;
--con7_29 : out  STD_LOGIC ;
--con7_30 : out  STD_LOGIC ;
--con7_31 : out  STD_LOGIC ;
--con7_32 : out  STD_LOGIC ;

--con8_05 : out  STD_LOGIC ;
--con8_06 : out  STD_LOGIC ;
--con8_07 : out  STD_LOGIC ;
--con8_08 : out  STD_LOGIC ;
--con8_11 : out  STD_LOGIC ;
--con8_12 : out  STD_LOGIC ;
--con8_13 : out  STD_LOGIC ;
--con8_14 : out  STD_LOGIC ;
--con8_17 : out  STD_LOGIC ;
--con8_18 : out  STD_LOGIC ;
--con8_19 : out  STD_LOGIC ;
--con8_20 : out  STD_LOGIC ;
--con8_23 : out  STD_LOGIC ;
--con8_24 : out  STD_LOGIC ;
--con8_25 : out  STD_LOGIC ;
--con8_26 : out  STD_LOGIC ;
--con8_29 : out  STD_LOGIC ;
--con8_30 : out  STD_LOGIC ;
--con8_31 : out  STD_LOGIC ;
--con8_32 : out  STD_LOGIC ;

--con9_05 : out  STD_LOGIC ;
--con9_06 : out  STD_LOGIC ;
--con9_07 : out  STD_LOGIC ;
--con9_08 : out  STD_LOGIC ;
--con9_11 : out  STD_LOGIC ;
--con9_12 : out  STD_LOGIC ;
--con9_13 : out  STD_LOGIC ;
--con9_14 : out  STD_LOGIC ;
--con9_17 : out  STD_LOGIC ;
--con9_18 : out  STD_LOGIC ;
--con9_19 : out  STD_LOGIC ;
--con9_20 : out  STD_LOGIC ;
--con9_23 : out  STD_LOGIC ;
--con9_24 : out  STD_LOGIC ;
--con9_25 : out  STD_LOGIC ;
--con9_26 : out  STD_LOGIC ;
--con9_29 : out  STD_LOGIC ;
--con9_30 : out  STD_LOGIC ;
--con9_31 : out  STD_LOGIC ;
--con9_32 : out  STD_LOGIC 

   );
end internal_leds_pushbuttons;






 
architecture Behavioral of internal_leds_pushbuttons is


-- configuration constants
constant counter_width : integer := 25 ; --  we could get a minimum frequency of 25 MHz / ( 2 ** 25 ) = 0.74 Hz
                                         --  or 1.34 seconds.
constant counter_top_count_integer : integer :=  integer(0.25 * 25_000_000) ; -- 0,25 second period 


 
constant counter_top_count : unsigned(counter_width - 1 downto 0) 
                              := to_unsigned(counter_top_count_integer, counter_width)  ; 




 
-- counter
signal counter_register : unsigned(counter_width - 1 downto 0) ; 
signal counter_reg_next : unsigned(counter_width - 1 downto 0) ; 
signal counter_max      : std_logic ;
signal counter_max_next : std_logic ;

-- internal led counter
signal led_reg      : unsigned ( 3 downto 0 ) ;
signal led_reg_next : unsigned ( 3 downto 0 ) ;


signal buttom_pressed : std_logic ; -- any pushbutton pressed
signal switch_on      : std_logic ; -- any switch in on position






begin

-- make sure counter_top_count_integer fits within the max value permited by counter
--   If necessary increase counter_width constant  
assert ( counter_top_count_integer <=  ( ( 2 ** counter_width ) -1 ) ) 
        REPORT "counter_top_count_integer TOO high" SEVERITY  failure  ;


-- counter
--   issues just one pulse (one clk_25mhz cycle) in counter_max whenever counter_reg reaches counter_top_count 

-- counter - registered logic
process (clk_25mhz , reset_pow_on) 
begin
   if reset_pow_on = '1' then
      counter_register <= (others => '0') ;
      counter_max      <= '0' ;
   elsif rising_edge (clk_25mhz) then
      counter_register <= counter_reg_next ;
      counter_max      <= counter_max_next ;
   end if ;
end process ;

-- counter - combinatorial logic
process (counter_register)
begin
   if counter_register = counter_top_count  then
      counter_max_next <= '1' ;
      counter_reg_next <= (others => '0') ;
   else
      counter_max_next <= '0' ;
      counter_reg_next <= counter_register + 1 ;
   end if;
end process ;







-- led_reg (4 bit binary counter)
-- led reg - registered logic
--    register is updated only when counter_max is active
process (clk_25mhz , reset_pow_on) 
begin
   if reset_pow_on = '1' then
      led_reg <= (others => '0') ;
   elsif rising_edge (clk_25mhz) then
      if counter_max = '1' then
      led_reg <= led_reg_next ;
      end if;
   end if ;
end process ;

-- led reg - commbinatorial logic
process ( led_reg )
begin
   led_reg_next <= led_reg + 1 ;
end process ;






-- control of leds
--    when any pushbutton is pressed each led reflects the state of pushbutton 
--      otherwise
--         when switch is on each led reflects the state of the switchs 
--           otherwise 
--             the leds follow the state of the led_reg counter

buttom_pressed <= '1' when (push_buttom_1_n = '0') or (push_buttom_2_n = '0') or (push_buttom_3_n = '0') or (push_buttom_4_n = '0') else
                  '0' ;

switch_on   <= '1' when ( switch_1_n = '0' ) or ( switch_2_n = '0' ) or ( switch_3_n = '0' ) or ( switch_4_n = '0' )  else
               '0' ;

 
led1_n <= push_buttom_1_n when buttom_pressed = '1'  else
          switch_1_n      when switch_on      = '1'  else
          not led_reg(0) ;
led2_n <= push_buttom_2_n when buttom_pressed = '1'  else
          switch_2_n      when switch_on      = '1'  else
          not led_reg(1) ;
led3_n <= push_buttom_3_n when buttom_pressed = '1'  else
          switch_3_n      when switch_on      = '1'  else
          not led_reg(2) ;
led4_n <= push_buttom_4_n when buttom_pressed = '1'  else
          switch_4_n      when switch_on      = '1'  else
          not led_reg(3) ;


-- uart test
--    it implements a simple "loop back" test 

CONSOLE_RTS_N <= CONSOLE_CTS_N when CONSOLE_USB_PRESENT = '1' else
                 '1' ;
        
CONSOLE_TXD   <= CONSOLE_RXD   when CONSOLE_USB_PRESENT = '1' else
                 '1' ;
                             

end Behavioral;

